#if !defined(__DJGPP__)

#include <stdio.h>
#include <assert.h>
#include <dos.h>
//#include <pldos32.h>

#define __EXC_INTERNAL
#include <exc.h>

/*-------------------------------------------------------------------*/

char * page_type (USHORT pg)
{
  static char buf[80];
  char *p = buf;

  p += sprintf (p,"pt=%04Xh: %s", pg, pg & PTYP_ALLOC ? "/alloc" : "/unmap");
  if (pg & PTYP_PHYSDEV) p += sprintf (p,"/phys");
  if (pg & PT_LOCKED)    p += sprintf (p,"/lock");
  if (pg & PT_SWAPPED)   p += sprintf (p,"/swap");
  if (pg & PT_ROFILE)    p += sprintf (p,"/rof");
  if (pg & PT_RWFILE)    p += sprintf (p,"/rwf");
  return (buf);
}

char * pte_info (ULONG pte)
{
  static char buf[80];
  char *p = buf;

  p += sprintf (p,"pte=%08Xh: ",pte);
  if (pte & PE_DIRTY   ) p += sprintf (p,"/dirty");
  if (pte & PE_ACCESSED) p += sprintf (p,"/acc");
  if (pte & PE_PCD     ) p += sprintf (p,"/ca-");
  if (pte & PE_PWT     ) p += sprintf (p,"/cw+");
  if (pte & PE_USER    ) p += sprintf (p,"/usr");
  if (pte & PE_WRITE   ) p += sprintf (p,"/wri");
  if (pte & PE_PRESENT ) p += sprintf (p,"/pre");
  return (buf);
}

/*-------------------------------------------------------------------*/

char * descr_access (UCHAR acc)
{
  static char buf[80];
  char *p = buf;

  p += sprintf (p,"%02Xh: ",acc);
  if (acc & AR_ACCESSED) p += sprintf (p,"/acc");
  if (acc & AR_CREAD)    p += sprintf (p,"/rd");
  if (acc & AR_CCONF)    p += sprintf (p,"/co");
  if (acc & AR_CSEG)     p += sprintf (p,"/cs");
  if (acc & AR_DWRITE)   p += sprintf (p,"/dw");
  if (acc & AR_EUP)      p += sprintf (p,"/up");
  if (acc & AR_EDOWN)    p += sprintf (p,"/dn");
  if (acc & AR_PRESENT)  p += sprintf (p,"/pr");
  if (acc & AR_SYS)      p += sprintf (p,"/sys");
  if (acc & AR_USER)     p += sprintf (p,"/usr");

  p += sprintf (p,", dpl=%u", (acc & AR_DPLM) >> AR_DPLSC);
  return (buf);
}

/*-------------------------------------------------------------------*/

void print_descr (USHORT sel, CD_DES *descr)
{
  ULONG limit,base;

  base   = descr->base0_15;
  base  += (ULONG)(descr->base16_23 << 16);
  base  += (ULONG)(descr->base24_31 << 24);

  limit  = descr->limit0_15;
  limit += (ULONG)((descr->limit16_19 & D6_HLIM) << 16);
  if (descr->limit16_19 & SG_PAGE)
       limit = (limit << 12) + 0x1000;
  else limit++;
  printf ("\ndescr  %02Xh: base %08Xh (%u Kb), limit %08Xh (%u kb)\naccess %s\n",
          sel,base,base >> 10, limit,limit >> 10, descr_access(descr->arights));
}
/*-------------------------------------------------------------------*/

int main (void)
{
  #define NP  5     /* # of pages */
  UINT   sel,i,loop;
  CD_DES descr;
  FARPTR fp;

  assert (!_dos_allocmem(NP,&sel));
  printf ("Selector %02Xh created\n",sel);

  assert (!_dx_ldt_rd(sel,(UCHAR*)&descr));  /* get descriptor        */
  print_descr (sel,&descr);

  descr.base0_15   += 4096;                  /* shift base page 1 up  */
  descr.limit16_19 |= SG_PAGE | DOS_32;      /* page granular segment */
  descr.arights     = AR_DATA | AR_DPL3;
  assert (!_dx_ldt_wr(sel,(UCHAR*)&descr));  /* update descriptor     */
  assert (!_dx_ldt_rd(sel,(UCHAR*)&descr));  /* read it back          */

  print_descr (sel,&descr);

  puts ("\ncontent:");
  FP_SET (fp,0,sel);

  for (i=0; i<50; i++)
  {
    printf (" %02X",PeekFarByte(fp));
    if (i == 24) puts ("");
    FP_INCR (fp,1);
  }

  assert (!_dx_page_lock(fp+(1<<12),1));           /* lock page 1 */
//assert (!_dx_map_pgs(fp+(2<<12),1<<12,0xB8000)); /* map page 2 to screen */

  for (loop = 0; loop < 2; loop++)
  {
    USHORT pgType[NP];

    puts ("\n-------------------------------------"\
          "---------------------------------------");

    assert (!_dx_ldt_rd(sel,(UCHAR*)&descr));
    print_descr (sel,&descr);

    FP_SET (fp,0,sel);
    _dx_gtyp_pgs (fp, NP<<12, (UCHAR*)&pgType);
    puts ("Page info:");

    for (i=0; i<NP; i++)
    {
      ULONG pte,ptInfo;
      _dx_rd_ptinf (fp + (i << 12), &pte,&ptInfo);
      printf ("Page %d: %-30.30s %-30.30s\n",
              i,page_type(pgType[i]),pte_info(pte));
    }
    sel = My_CS();
  }
  return (0);
}

#endif /* !__DJGPP__ */
