/* Copyright (C) 1998 DJ Delorie, see COPYING.DJ for details */
/* Copyright (C) 1997 DJ Delorie, see COPYING.DJ for details */
/* Copyright (C) 1995 DJ Delorie, see COPYING.DJ for details */
/*
** Copyright (C) 1993 DJ Delorie, 24 Kirsten Ave, Rochester NH 03867-2954
**
** This file is distributed under the terms listed in the document
** "copying.dj", available from DJ Delorie at the address above.
** A copy of "copying.dj" should accompany this file; if not, a copy
** should be available from where this file was obtained.  This file
** may not be distributed without a verbatim copy of "copying.dj".
**
** This file is distributed WITHOUT ANY WARRANTY; without even the implied
** warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*/

/* Modified by Charles Sandmann 1995 for DJGPP V2 (bug fixes) 
   incorporate changes by Morten Welinder, terra@diku.dk */

/* Modified and reduced by Gisle Vanem <giva@bgnett.no> 1998
   for libexc.a */

#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <unistd.h>
#include <io.h>
#include <libc/unconst.h>

#include <coff.h>
#include <debug/syms.h>
#include <debug/stab.h>
#include <debug/wild.h>

#include "../trace.h"

static int undefined_symbol = 0;
static int syms_printwhy    = 0;


/* built internally
 */
static FileNode *files;
static int      num_files;

typedef struct SymNode {
        char  *name;
        ULONG  address;
      } SymNode;

static SymNode *syms;
static SymNode *syms_byname;
static int      num_syms;

static int syms_sort_bn (const void *a, const void *b)
{
  const SymNode *sa = (const SymNode *)a;
  const SymNode *sb = (const SymNode *)b;
  return strcmp (sa->name, sb->name);
}

static int syms_sort_bv (const void *a, const void *b)
{
  const SymNode *sa = (const SymNode *)a;
  const SymNode *sb = (const SymNode *)b;
  return (sa->address - sb->address);
}

static char *symndup (char *s, int len)
{
  char *rv = malloc (len+1);

  if (!rv)
     return (NULL);
  memcpy (rv,s,len);
  rv[len] = 0;
  return (rv);
}


void syms_init (char *fname)
{
  FILE *fd = fopen(fname, "rb");

  if (!fd)
     perror (fname);
  else
  {
    process_file (fd, 0);

    syms_byname = malloc (num_syms * sizeof(SymNode));
    memcpy (syms_byname, syms, num_syms * sizeof(SymNode));
    qsort (syms_byname, num_syms, sizeof(SymNode), syms_sort_bn);
    qsort (syms, num_syms, sizeof(SymNode), syms_sort_bv);
    fclose (fd);
  }
}

static char noname_buf[11];

char *syms_val2name (ULONG val, ULONG *delta)
{
  int above, below, mid;

  if (delta)
     *delta = 0;

  if (num_syms <= 0)
     goto noname;

  above = num_syms;
  below = -1;
  mid = 0;			/* Or complains about maybe uninit */

  while (above-below > 1)
  {
    mid = (above+below)/2;
    if (syms[mid].address == val)
       break;
    if (syms[mid].address > val)
         above = mid;
    else below = mid;
  }
  if (syms[mid].address > val)
  {
    if (mid == 0)
       goto noname;
    mid--; /* the last below was it */
  }
  if (mid < 0)
     goto noname;

  if (!strcmp(syms[mid].name, "_end") ||
      !strcmp(syms[mid].name, "__end") ||
      !strcmp(syms[mid].name, "_etext"))
     goto noname;

  if (delta)
     *delta = val - syms[mid].address;
  return (syms[mid].name);

noname:
  sprintf (noname_buf, "%#lx", val);
  return (noname_buf);
}


/* Code added by MW, changed by CWS
 */
char *syms_module (int no)
{
  if (no < 0 || no >= num_files)
    return (0);
  return files[no].filename;
}

ULONG syms_line2val (char *filename, int lnum)
{
  int f,l;

  for (f = 0; f < num_files; f++)
    if (!strcmp (filename, files[f].filename))
    {
      for (l = 0; l < files[f].num_lines; l++)
	if (files[f].lines[l].l_lnno == lnum)
	  return files[f].lines[l].l_addr.l_paddr;
      return (0);
    }
  return (0);
}
