TITLE High Resolution Timer for PharLap 386|DosX
NAME  HiResTimer

INCLUDE exc.inc

_DATA SEGMENT
      clk_init  db  0
_DATA ENDS

_TEXT SEGMENT

;----------------------------------------------------------------------------
; Descr:  Get # of mSec. Good to approx 14 minutes timings
; Input:  None
; Output: EAX
; Regs changed: ECX, EDX
;----------------------------------------------------------------------------

C_PROC clock
       call GetHiResTimer         ; get ticks in EAX
       xor  edx, edx              ; dividend = 0:EAX
       mov  ecx, 1193             ; divisor = ECX
       div  ecx                   ; EAX = result, EDX = remainder
       ret


;----------------------------------------------------------------------------
; Descr:  return timer value as DWORD in EAX (actually only 6 bytes)
; Input:  None
; Output: EAX
; Regs changed:
;----------------------------------------------------------------------------

GetHiResTimer:
       cmp  clk_init, 0
       je   @init_clk
@cont_clk:
       push ebx
       xor  eax, eax              ; clear result
       out  43h, al               ; latch counter 0
       in   al, 40h               ; read lsb
       mov  ah, al                ; hide LSB in AH
       in   al, 40h               ; read msb
       xchg al, ah
       not  eax                   ; invert AX (counter counts down)
       mov  ebx, SS_DOSMEM        ; selector for 0 - 1Mbyte range
       push es
       mov  es, bx
       mov  ebx, es:[46Ch]        ; number of 18.2 ticks since midnight
       shl  ebx, 16               ; make room for AX
       add  eax, ebx              ; number of 65535*18.2 ticks/sec
       pop  es                    ; approx 1193 in 1mSec
       pop  ebx
       ret

@init_clk:
       call SetHiResTimer
       mov  clk_init, 1
       xor  eax, eax              ; 1st time called, return 0
       jmp  short @cont_clk

;----------------------------------------------------------------------------
; Descr: sets timer 1 to update 65535*18.2 per second (1.193180 MHz)
;        Does not affect INT 8 or memory refresh rate
; Input:  None
; Output: None
; Regs changed: EAX
;----------------------------------------------------------------------------

SetHiResTimer:
       mov al, 00110100b          ; timer 1 in mode 2
       out 43h, al
       mov al, 0
       out 40h, al
       out 40h, al
       ret

_TEXT ENDS

END


