/*
 *  Installation of pmode & rmode interrupt handlers.
 *  Only for 32-bit C/C++ and PharLap DosX
 *
 *  by G. Vanem 20-Sep-97
 */

#define __EXC_INTERNAL
#include "exc.h"
#include "trace.h"

#define CALLBACK_STACK_SIZE (16*1024)  /* 16 kByte */

int _dx_install_intr (rmodeIntr  rmHandler,
                      pmodeIntr  pmHandler,
                      UINT       intr_num,
                      PMRM_INTR *intr)
{
  if (intr && intr->number)   /* already installed */
     return (1);

  if (rmHandler)
  {
    REALPTR cb = _dx_alloc_rmode_wrapper_iret ((pmodeHook)rmHandler,
                                               CALLBACK_STACK_SIZE);
    if (!cb)
       return (0);

    if (intr)
       _dx_rmiv_get (intr_num, &intr->rmVector);
    _dx_rmiv_set (intr_num, cb);
    intr->callBack = cb;
  }

  if (pmHandler)
  {
    FARPTR fp;

    FP_SET (fp, pmHandler, My_CS());
    if (intr)
       _dx_pmiv_get (intr_num, &intr->pmVector);
    _dx_pmiv_set (intr_num, fp);
  }
  if (intr)
      intr->number = intr_num;

//_dx_dbg_regsav (1); // !! test
  return (1);
}

/*-------------------------------------------------------------------*/

void _dx_cleanup_intr (PMRM_INTR *intr)
{
  if (intr->number)
  {
#if (__CMPLR_FEATURES__ & __FEATURE_FARPTR__)
    if (intr->pmVector)
       _dx_pmiv_set (intr->number, intr->pmVector);
    intr->pmVector = 0;
#else
    if (intr->pmVector.Off)
       _dx_pmiv_set (intr->number, intr->pmVector);
    intr->pmVector.Sel = 0;
    intr->pmVector.Off = 0;
#endif

    if (intr->rmVector)
       _dx_rmiv_set (intr->number, intr->rmVector);

    if (intr->callBack)
       _dx_free_rmode_wrapper (intr->callBack);

    intr->number   = 0;
    intr->rmVector = 0;
    intr->callBack = 0;

    _dx_dump_intr();   // !! test
  }
}

/*-------------------------------------------------------------------*/

int _dx_request_irq (int         irq_num,
                     IrqHandler *irq_handler,
                     int         irq_flags,
                     char       *irq_name,
                     void       *irq_id)
{
  ARGSUSED (irq_num); // !! to-do
  ARGSUSED (irq_handler);
  ARGSUSED (irq_flags);
  ARGSUSED (irq_name);
  ARGSUSED (irq_id);
  return (0);
}

/*-------------------------------------------------------------------*/

void _dx_dump_regs (SWI_REGS *reg)
{
  char *sym;

  printf ("Regs: EAX %08X  EBX %08X  ECX %08X  EDX %08X\n"
          "      ESI %08X  EDI %08X  EBP %08X  ESP %08X\n"
          "      DS  %04X      ES  %04X    SS  %04X\n"
          "      FS  %04X      GS  %04X    FLG %04X\n"
          "      CS:EIP %02X:%08X",
          reg->eax, reg->ebx, reg->ecx, reg->edx,
          reg->esi, reg->edi, reg->ebp, reg->esp,
          reg->ds,  reg->es,  reg->ss,  reg->fs, reg->gs, reg->flags,
          reg->cs,  reg->eip);

  sym = GetSymbolStr (reg->eip);
  if (*sym)
       printf (" (%s)\n", sym);
  else printf ("\n");
}

int _dx_dump_intr (void)
{
  SWI_REGS regs, cc_regs;
  ULONG    handle;
  ULONG    int_num;
  BOOL     next  = 0;
  int      loops = 0;

  while (_dx_swi_rdregs (&regs,next,&handle,&int_num) == 0)
  {
    next = 1;
    printf ("\nstate %d, handle %lu, intr %02lX\n",
            ++loops, handle,  int_num);
    _dx_dump_regs (&regs);
  }

  if (_dx_cc_rdregs(&cc_regs) == 0 && cc_regs.eip != regs.eip)
  {
    printf ("\nCtrl-C state:\n");
    _dx_dump_regs (&cc_regs);
  }
  return (loops);
}

