PAGE  60,132
TITLE Pmode signal ISRs
NAME  signal_isr

INCLUDE EXC.INC

EXTRN @_context       : ExcStruc
EXTRN @_oldintr_8     : PMRM_INTR
EXTRN @_oldintr_9     : PMRM_INTR
EXTRN @_SigInt_8      : near
EXTRN @_SigInt_9      : near
EXTRN @_ExcFillContext: near

_DATA SEGMENT
      int_stack_8    dd 1024 dup (?)
      stack_start_8  label dword

      int_stack_9    dd 1024 dup (?)
      stack_start_9  label dword

      fanChars       db '-\|/'    ; debug
      fanIndex       dd 0
_DATA ENDS


SHOW_FAN MACRO
      push es
      mov  ax, SS_SCREEN
      mov  es, ax
      mov  edi, 156               ;; upper right corner - 1
      mov  ebx, fanIndex
      mov  al, fanChars[ebx]      ;; get write char
      mov  ah, 15                 ;; color = white on black
      stosw                       ;; write to screen at ES:EDI
      inc  fanIndex               ;; update next index
      and  fanIndex, 3
      pop  es
ENDM

_TEXT SEGMENT

ISR_GLUE MACRO isr
      pushad                      ;; save all general registers
      pushfd
      push ds
      push es
    ; sti                         ;; enable interrupts
      cld                         ;; C code need direction flag off

      mov  cx, cs
      add  cx, 8
      mov  ds, cx                 ;; DS = CS + 8
      mov  es, cx                 ;; ES = DS

comment ~
      mov  eax, [esp+30h+44]      ;; get DosX flags from the stack
      test eax, 3                 ;; LOADSEG and RMODE bit clear ?
      jnz  @exit_&isr             ;; no, exit

      mov  ebx, @_oldintr_&isr&.rmVector ;; address of old r-mode INT 8
      mov  ecx, 1                        ;; 1 word from p-stack to r-stack
                                         ;; emulates an r-mode int
      mov  eax, 250Eh                    ;; call r-mode procedure
      int  21h
~
      push eax                    ;; push eax/ebx so context gets right
      push ebx
      mov  ebx, esp
      add  ebx, 4*10+8
      call @_ExcFillContext       ;; setup context for signal-handler
      mov  @_context.excNum, (NUM_EX-8+&isr)

                                  ;; int 8 -> SIGALRM = 18
                                  ;; int 9 -> SIGINT  = 19
      add  esp, 2*4               ;; pop eax/ebx
      mov  ecx, esp               ;; get current stack pointer
      mov  dx, ss                 ;; and selector
      mov  ax, ds
      mov  ss, ax                 ;; new SS = DS
      lea  esp, stack_start_&isr  ;; new ESP = end of data array
      push edx
      push ecx                    ;; save old SS,ESP on new stack

      SHOW_FAN                    ;; !! debug

      call @_SigInt_&isr&         ;; do signal processing
      lss  esp,pword ptr ss:[esp] ;; restore old stack
@exit_&isr:
      pop  es
      pop  ds
      popfd
      popad
ENDM


;-----------------------------------------------------------------------
;
; INT 8 handler for SIGALRM/SIGUSR1 processing. Installed from signal.c

C_PROC SigInt_8pm

       ISR_GLUE 8
       iretd

;-----------------------------------------------------------------
;
; INT 9 handler for SIGINT processing. Installed from signal.c

C_PROC SigInt_9pm

       ISR_GLUE 9
       iretd

_TEXT ENDS
END

